﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PojeClient {
    // 3手以内で打てる量を最大にするような，発動しない初手を選ぶ．一定の条件下で実際に攻撃する．
    class AI_ChargeAndFire3 : Player {
        private int fMyIndex;
        private int fTimeLimit;
        private Action[] fAnswerBox;
        private Random fRand;

        public String getName() {
            return "AI : Charge and Fire III";
        }

        public void newGame(int yourIndex) {
            fRand = new Random(yourIndex + (new Random()).Next());
            fMyIndex = yourIndex;
        }

        public void endGame(int result) {
        }

        public void setParam(int timeLimit, Action[] answerBox) {
            fTimeLimit = timeLimit;
            fAnswerBox = answerBox;
        }

        public void makeAction(object currentState) {
            State state = (State)currentState;
            Board myBoard = state.getBoard(fMyIndex);
            Board oppBoard = state.getBoard(1 - fMyIndex);
            List<Action> possibleActions = myBoard.createPossibleActions();

            int restTurn = int.MaxValue;
            // Case 1. 埋まりそうな場合．
            if (AITools.getRestCount(myBoard) < 17) restTurn = 3;
            if (AITools.getRestCount(myBoard) < 15) restTurn = 2;
            if (AITools.getRestCount(myBoard) < 13) restTurn = 1;
            // Case 2. 相手の攻撃がある程度あって近い場合．
            JamInfo hisAttack = AITools.calcOpponentsAttack(oppBoard, myBoard.getJamInfo(), state.getTurn());   // 今自分の上にある　＋　相手が連鎖中のもの　を含めた邪魔珠．
            hisAttack.deleteJam(oppBoard.getJamInfo().getTotalCount());                          // そこから，今相手の上にあるものを引く．これで実質攻撃される数と時間が分かる．

            if (hisAttack.getCountTill(state.getTurn() + 3) > 5) restTurn = 3;
            if (hisAttack.getCountTill(state.getTurn() + 2) > 5) restTurn = 2;
            if (hisAttack.getCountTill(state.getTurn() + 1) > 5) restTurn = 1;

            //// 危機が３手以内に迫っているなら，打てる中で最も攻撃力の高い手を打つ．(Fire)
            if (restTurn <= 3) {
                AITool_Result result2 = AITools.calcBestAttackAction(myBoard, possibleActions, restTurn);
                if (result2.Gain + result2.JamVanishCount > 0) {
                    fAnswerBox[0] = result2.Action;
                } else {
                    // 消せないなら，ランダムに置く
                    fAnswerBox[0] = possibleActions[fRand.Next(possibleActions.Count)];
                }
                return;
            }


            // そうでないなら，将来の攻撃力をベースに，それが同じなら適当にくっつくように，消さない手を打つ． (Charge)
            int bestScore = int.MinValue;
            Action bestAction = null;
            foreach (Action a in possibleActions) {
                int score;
                AITool_Result result = AITools.calcTotalGain(myBoard, a, false);         // その手の攻撃結果
                if (result.IsVanish == true) {
                    // 消してしまう場合はそれでも小さいものを選ぶ
                    score = -result.Gain;
                    if (result.Gain > 250) score = 200*result.Gain;   // （このクラスとしては）十分大きい攻撃ならしてしまう
                    if (result.Gain < hisAttack.getTotalCount()) score = -result.Gain;     // ただそれでも負けるようならやっぱり攻撃しないで粘る
                } else {
                    int connects = AITools.calcConnectCount(myBoard, a);  // その手がどれだけ周りの石と同色か
                    // 消去なし落下後のポテンシャルを得る．Nextはあと2個分見えている
                    AITool_Result potential = AITools.calcBestAttackAction(result.NextBoard, result.NextBoard.createPossibleActions(), Settings.NextOpenTurns - 1);

                    // ポテンシャルを主に，適当にくっつくようなスコア計算
                    score = fRand.Next(100) + 70 * AITools.calcConnectCount(myBoard, a) + 200 * potential.Gain;
                }
                if (score > bestScore) {
                    bestScore = score;
                    bestAction = a;
                }
            }

//            Logger.log("score:" + bestScore / 200);

            fAnswerBox[0] = bestAction;
        }
    }
}
