﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PojeClient {
    // AI作成のテンプレート．
    class AI_template : Player {
        private int fMyIndex;        // 自分が fStateのfBoardsの [0]か[1]か
        private int fTimeLimit;      // 何秒以内に返事しないといけないか
        private Action[] fAnswerBox; // fAnserBox[0] に入っている Actionが実行される
        private Random fRand;        // 乱数生成器

        public String getName() {
            return "（プログラム名）";
        }

        public void newGame(int yourIndex) {
            fRand = new Random(yourIndex + (new Random()).Next()); // 適当に乱数を初期化
            fMyIndex = yourIndex;
        }

        public void endGame(int result) {
            // ゲーム終了時になにか処理したければここに記述する
        }

        public void setParam(int timeLimit, Action[] answerBox) {
            fTimeLimit = timeLimit;
            fAnswerBox = answerBox;
        }

        // 自分の操作を決めて fAnswerBox[0] に格納する
        public void makeAction(object currentState) {
            State state = (State)currentState;                               // 現在の状態全部
            Board myBoard = state.getBoard(fMyIndex);                        // 自分の盤面
            Board oppBoard = state.getBoard(1 - fMyIndex);                   // 相手の盤面
            List<Action> possibleActions = myBoard.createPossibleActions();  // 自分が取りうる行動の集合

            // 盤上の残りスペースはこのように取る
            int restSpace = AITools.getRestCount(myBoard);
            
            // 相手から攻撃を受けている場合のダメージ量はこのように計算する
            JamInfo hisAttack = AITools.calcOpponentsAttack(oppBoard, myBoard.getJamInfo(), state.getTurn());   // 今自分の上にある　＋　相手が連鎖中のもの　を含めた邪魔珠．
            hisAttack.deleteJam(oppBoard.getJamInfo().getTotalCount());                          // そこから，今相手の上にあるものを引く．これで実質攻撃される数と時間が分かる．
            
            int totalDamage = hisAttack.getTotalCount();                          // 攻撃力全部
            int damageComingSoon = hisAttack.getCountTill(state.getTurn() + 1);   // あと1ターンでこちらに降ってくる量


            // たとえば危機に対応して，1ターン内で打てる中で最も攻撃力の高い手を打つならこうする
            if (damageComingSoon > 10) {
                AITool_Result result2 = AITools.calcBestAttackAction(myBoard, possibleActions, 1);
                if (result2.Gain + result2.JamVanishCount > 0) {
                    // 少しでも消せるなら
                    fAnswerBox[0] = result2.Action;
                } else {
                    // 消せないなら，ランダムに置く
                    fAnswerBox[0] = possibleActions[fRand.Next(possibleActions.Count)];
                }
                return;
            }


            // 各合法手に点数付けして，その中からベストのものを選びたい場合の書き方
            int bestScore = int.MinValue;
            Action bestAction = null;
            foreach (Action a in possibleActions) {    // 全ての合法手
                int score = 0;
                AITool_Result result = AITools.calcTotalGain(myBoard, a, false);         // その手の攻撃結果

                // 例えば，100個以上攻撃できるなら高得点をあげる
                if (result.Gain > 100) {
                    score = result.Gain * 10000;
                }
                // 例えば，100個未満の攻撃は低得点にしたいなら，
                if (result.IsVanish == true && result.Gain < 100) {
                    score = -10000;
                }
                // 例えば，同じ色でくっつくのを推奨したいなら，
                if (result.IsVanish == false) {
                    int connects = AITools.calcConnectCount(myBoard, a);                     // その手がどれだけ周りの石と同色か
                    score = connects * 100;
                }
                // 例えば，落下後のポテンシャル（あと２回でどれだけ攻撃できるか）を大きくしたいなら
                if (result.IsVanish == false) {
                    score = AITools.calcBestAttackAction(result.NextBoard, result.NextBoard.createPossibleActions(), 2).Gain * 100;
                }

                // 各手のスコアをログに吐きたいなら
                //Logger.log("score = " + score + " : " + a.toBriefString());

                // 例えば，同じスコアの手をばらけさせたいなら，
                score += fRand.Next(10);

                // など，適当に点数付けをしたうえで，現状の最善のものと比較する
                if (score > bestScore) {
                    bestScore = score;
                    bestAction = a;
                }
            }
            fAnswerBox[0] = bestAction;


            // もっと直接的に，自分や相手の盤上の珠の状態などが知りたければ，
            int stoneColor32 = myBoard.getStone(3, 2);   // 左から3列目，下から2段目の石の色．1-4なら色珠，0なら空所，-1なら邪魔珠
            int[] nexts = myBoard.getNextColors();       // 配珠の色．[0][1]が現在のもの，[2][3]が次，[4][5]がその次
            
        }
    }
    
}
