﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;


namespace PojeClient {
    public partial class Form_main : Form {
        private GameManager fGameManager;
        private SGFManager fSGFManager;
        private Player[,] fPlayerList;
        private bool fIsGameRunning;
        private int fRestTime;
        private VSServer fVSServer;



        public Form_main() {
            InitializeComponent();
        }


        // 起動後，１度だけやる処理はここにまとめる
        private void Form_main_Shown(object sender, EventArgs e) {
            // Logger の設定
            Logger.setLogBox(this, textBox_mainLog);

            // プレイヤーリストの作成．
            int numberOfAIs = 7; // ●ここにAIの候補数を書く
            fPlayerList = new Player[numberOfAIs+1, 2];
            fPlayerList[0, 0] = new Player_User(this);
            fPlayerList[0, 1] = new Player_PTP();
            for (int i = 0; i <= 1; i++) {
                //● ここに AI の候補を書く 順番はどうでもいい
                fPlayerList[1, i] = new AI_ChargeAndFire3();
                fPlayerList[2, i] = new AI_NoAttack();
                fPlayerList[3, i] = new AI_Random();
                fPlayerList[4, i] = new AI_ChargeAndFire();
                fPlayerList[5, i] = new AI_ChargeAndFire2();
                fPlayerList[6, i] = new AI_ChargeAndFire3();
                fPlayerList[7, i] = new AI_template();
            }

            // 選択用 ComboBoxへの名前代入
            for (int i = 0; i <= numberOfAIs; i++) {
                comboBox_player1.Items.Add(fPlayerList[i, 0].getName());
                comboBox_player2.Items.Add(fPlayerList[i, 1].getName());
            }
            comboBox_player1.SelectedIndex = 0; // 左側のデフォルトはユーザ
            comboBox_player2.SelectedIndex = 1; // 右側のデフォルトは1番目のAI

            // 棋譜マネージャの生成
            fSGFManager = new SGFManager();

            // ゲームマネージャの生成と必要データ渡し
            fGameManager = new GameManager(this);
            fGameManager.setPicBoxes(picBox_board1, picBox_board2, picBox_next1, picBox_next2);
            fGameManager.setLabels(Label_Timer, Label_Turn);
            fGameManager.setSGFManager(fSGFManager);

        }

        private void mainMenu_Help_Click(object sender, EventArgs e) {
            MessageBox.Show("本プログラムはJAIST杯 崩珠大会，および学術研究のために制作・配布されたものです．\n" +
                            "営利目的での転用・再配布や，セガ社が権利を所有する\n" +
                            "“ぷよぷよ”との混同を生じさせる改変等はしないでください．\n" +
                            "また，本プログラムを用いたことによるあらゆる損害の補償はできません．\n\n" +
                            "　　　　　　　2011年 2月 24日　北陸先端科学技術大学院大学　池田 心");
        }

        // フォームの状態を，ゲーム中／通常　で切り替える
        private void setGameRunning(bool b) {
            fIsGameRunning = b;
            if (b == true) {
                mm_Start.Text = "Stop";
            } else {
                mm_Start.Text = "Start";
            }
            // だいたいのものは試合中は触ってほしくない
            sm_Load.Enabled = !b;
            sm_SaveAs.Enabled = !b;
            mm_startAsServer.Enabled = (!b) & Settings.enableStartAsServer;
            comboBox_player1.Enabled = !b;
            comboBox_player2.Enabled = !b;
            // ゲーム進行関係，投了は例外
            mm_resign.Enabled = b;
        }

        private void mm_Start_Click(object sender, EventArgs e) {
            if (mm_Start.Text.Equals("Start") == true) {
                //ゲーム開始ボタンを中断ボタンに変更するなど，画面上の状態を変更
                setGameRunning(true);

                //選択されてるプレイヤーをゲームマネージャに渡してゲーム開始
                fGameManager.setPlayers(fPlayerList[comboBox_player1.SelectedIndex, 0], fPlayerList[comboBox_player2.SelectedIndex, 1]);

                fGameManager.executeGame(GameManager.PLAYTYPE_NORMAL);

                // 画面上の状態を戻す
                setGameRunning(false);
            } else {
                fPushedKey = Player_User.KEY_STOP;
            }
        }

        // 名前をつけて保存
        private void sm_SaveAs_Click(object sender, EventArgs e) {
            saveDialog.Filter = "*.sgf|*.sgf";
            if (saveDialog.ShowDialog() == DialogResult.OK) {
                fSGFManager.setFileName(saveDialog.FileName);
                fSGFManager.saveToFile();
            }
        }

        // ゲームをリプレイする．
        private void sm_Load_Click(object sender, EventArgs e) {
            openDialog.InitialDirectory = Application.StartupPath;
            openDialog.Filter="*.sgf|*.sgf";
            if (openDialog.ShowDialog() == DialogResult.OK) {
                if (fSGFManager.loadFromFile(openDialog.FileName) == true) {
                    setGameRunning(true);
                    fGameManager.executeGame(GameManager.PLAYTYPE_REPLAY);
                    setGameRunning(false);
                } else {
                    MessageBox.Show("Error: file unknown format?");
                }
            }

        }


        private void mm_resign_Click(object sender, EventArgs e) {
            fPushedKey = Player_User.KEY_RESIGN;
        }


        #region ユーザがキー操作する場合がらみ

        private int fPushedKey;

        public int getPushedKey() {
            return fPushedKey;
        }

        public void clearPushedKey() {
            fPushedKey = Player_User.KEY_NONE;
        }

        // キーボード入力を取る．
        private void Form_main_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e) {
            if (fIsGameRunning == false) return;
            fPushedKey = Player_User.KEY_NONE;
            if (e.KeyCode == Keys.Left  || e.KeyCode == Keys.NumPad4) fPushedKey = Player_User.KEY_LEFT;
            if (e.KeyCode == Keys.Right || e.KeyCode == Keys.NumPad6) fPushedKey = Player_User.KEY_RIGHT;
            if (e.KeyCode == Keys.X     || e.KeyCode == Keys.Up || e.KeyCode == Keys.NumPad5) fPushedKey = Player_User.KEY_ROTRIGHT;
            if (e.KeyCode == Keys.Z                                                         ) fPushedKey = Player_User.KEY_ROTLEFT;
            if (e.KeyCode == Keys.Space || e.KeyCode == Keys.Down || e.KeyCode == Keys.NumPad2) fPushedKey = Player_User.KEY_DROP;

            e.IsInputKey = true;
        }

        // 操作のためのキー入力を無視する
        private void textBox_mainLog_KeyPress(object sender, KeyPressEventArgs e) {
            if (fIsGameRunning == false) return;
            e.Handled = true;
        }

        // タイマーのOn/Off
        public void setTimerEnabled(bool b, int rest) {
            timer_User.Enabled = b;
            if (b == true) {
                Label_Timer.Text = "Time : " + rest;
                fRestTime = rest;
            } else {
                Label_Timer.Text = "Time : ";
            }
        }





        #endregion

        // 設定を変更するウィンドウを開く
        private void mm_Setting_Click(object sender, EventArgs e) {
            (new Form_ChangeSettings()).ShowDialog();
            mm_startAsServer.Enabled = Settings.enableStartAsServer & (!fIsGameRunning);
        }

        // １秒ごとに刻まれる．時間が切れたら，今の場所で落とすキーを押したことにする
        private void timer_User_Tick(object sender, EventArgs e) {
            fRestTime--;
            if (fRestTime == -1) {
                fPushedKey = Player_User.KEY_DROP;
            } else {
                Label_Timer.Text = "Time : " + fRestTime;
            }
        }


        // サーバとしてゲームを開始する
        private void mm_startAsServer_Click(object sender, EventArgs e) {
            fVSServer = new VSServer();
            fGameManager.setVSServer(fVSServer);
            setGameRunning(true);
            fVSServer.listenStart();
            Logger.log("Waiting for 2 connections");
            // 受付を開始する．その途中にSTOPが押されたら終了する
            while (fVSServer.fStatus != VSServer.STATUS_READY) {
                Thread.Sleep(50);
                Application.DoEvents();
                if (fPushedKey == Player_User.KEY_STOP) {
                    fVSServer.listenEnd();
                    fVSServer.closeConnection();
                    Logger.log("Waiting stopped");
                    fVSServer.fStatus = VSServer.STATUS_DISABLED;
                    setGameRunning(false);
                    return;
                }
                if (fVSServer.fStatus == VSServer.STATUS_DISABLED) {
                    return;
                }
            }
            fVSServer.listenEnd();
            Logger.log("2 players joined");
            fGameManager.executeGame(GameManager.PLAYTYPE_SERVER);
            setGameRunning(false);
            fVSServer.closeConnection();

        }

        private void Form_main_FormClosed(object sender, FormClosedEventArgs e) {
            // 各地の while を抜けるためのシグナル
            Signal.FormClosed = true;

            if (fVSServer != null) {
                fVSServer.closeConnection();
            }
        }








        #region ログ吐きがらみ

        delegate void AppendTextCallback(string text);

        public void AppendText(String text) {
            if (textBox_mainLog.InvokeRequired) {
                AppendTextCallback d = new AppendTextCallback(AppendText);
                this.Invoke(d, new object[] {text});
            } else {
                textBox_mainLog.AppendText(text);
            }
        }
        #endregion

    }
}
