﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace PojeClient {
    // 邪魔珠がいつ・いくつ・どこに降るかという情報．
    class JamInfo {
        private int fFallIndex;                 // 次はどこから邪魔珠が降るか．
        private List<Point> fJamCountAndTurn;   // xには個数， yには落ちるターンが保存される

        // デフォルトコンストラクタ
        public JamInfo() {
            fFallIndex = 1;
            fJamCountAndTurn = new List<Point>();
        }


        // コピーコンストラクタ
        public JamInfo(JamInfo src) {
            fFallIndex = src.getFallIndex();
            fJamCountAndTurn = new List<Point>();

            List<Point> points = src.refJamCountAndTurn();
            foreach (Point p in points) {
                fJamCountAndTurn.Add(new Point(p.X, p.Y));
            }
        }

        public int getFallIndex() {
            return fFallIndex;
        }

        public void setFallIndex(int idx) {
            fFallIndex = idx;
        }

        private List<Point> refJamCountAndTurn() {
            return fJamCountAndTurn;
        }

        // 現在の珠数
        public int getTotalCount() {
            int count = 0;
            foreach (Point p in fJamCountAndTurn) {
                count += p.X;
            }
            return count;
        }

        // あるターンまでに降る珠数
        public int getCountTill(int turn) {
            int count = 0;
            foreach (Point p in fJamCountAndTurn) {
                if (p.Y <= turn) {
                    count += p.X;
                }
            }
            return count;
        }
 




        // ターン開始時に，落ちる珠の数を得て，リストからも抜く．
        public int pickJamDropCount(int turn) {
            // まず，落ちる数をカウントし，リストからも抜く．
            int count = 0;
            int idx = 0;
            while (idx < fJamCountAndTurn.Count) {
                Point p = fJamCountAndTurn[idx];
                if (p.Y <= turn) {
                    count += p.X;
                    fJamCountAndTurn.RemoveAt(idx);     // 同じインデックスを見る
                } else {
                    idx++;
                }
            }
            return count;
        }



        // 未確定の邪魔珠（あるなら最後にある）の，落ちるターンを定める
        // すでに相殺されてたり，４個消し１連鎖の場合もあるのでその場合は別になにもしない
        public void setJamTurn(int turn) {
            int idx = fJamCountAndTurn.Count - 1;
            if (idx == -1) return;
            if (fJamCountAndTurn[idx].Y != FUTURE) return;
            fJamCountAndTurn[idx] = new Point(fJamCountAndTurn[idx].X, turn);
        }
        
        // 今回連鎖した分（相手に送れる分）を，まず自分のJamを相殺するのに使う
        // 古いほうから相殺する
        public int deleteJam(int count) {
            // count が 0以上か， Jamがある限り続ける
            while (count > 0 && fJamCountAndTurn.Count > 0) {
                int j = fJamCountAndTurn[0].X;
                if (j > count) {
                    fJamCountAndTurn[0] = new Point(fJamCountAndTurn[0].X - count, fJamCountAndTurn[0].Y);
                    count = 0;
                    break;
                } else {
                    count -= fJamCountAndTurn[0].X;
                    fJamCountAndTurn.RemoveAt(0);
                }
            }
            return count;
        }

        // 今回連鎖した分（相手に送れる分）の，相殺した残りを，相手側に足す
        // 一番新しいものを見て，ターンが確定していれば前の分が残っているので新しく作る．そうでなければ加算する．
        public void addJam(int count) {
            if (count == 0) return;

            if (fJamCountAndTurn.Count == 0) {
                // ないので新しく作る
                fJamCountAndTurn.Add(new Point(count, FUTURE));
            } else {
                Point p = fJamCountAndTurn[fJamCountAndTurn.Count - 1];
                if (p.Y == FUTURE) {
                    // あって，ターン未確定なので，加算する
                    fJamCountAndTurn[fJamCountAndTurn.Count - 1] = new Point(p.X + count, p.Y);
                } else {
                    // あるけど，ターン確定なので，新しく作る
                    fJamCountAndTurn.Add(new Point(count, FUTURE));
                }
            }
        }


        // 文字列に直す
        public String toBriefString() {
            String s = "";
            foreach (Point p in fJamCountAndTurn) {
                if (p.Y == FUTURE) {
                    s = s + p.X + "+... ";
                } else {
                    s = s + p.X + " (t=" + p.Y + ") ";
                }
            }
            return s;
        }





        //--- static

        public const int FUTURE = 1000000;  // どのターンに落ちるかまだ分からない状態（ずっと先のターン）

        // c連鎖目，n個消しの場合の落ちる珠数
        public static int getGainJamCount(int chain, int vnum) {
            return Settings.JamCountByChain[chain] - Settings.JamCountByChain[chain - 1] + (vnum - Settings.NumVanish) * Settings.JamCountByAddition[chain];
        }
    }
}
