﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows.Forms;

namespace PojeClient {
    // サーバごしの相手
    // 参照   http://homepage2.nifty.com/nonnon/SoftSample/CS.NET/SampleTcpIp.html
    class Player_PTP : Player {


        private TcpClient fSocket;
        private NetworkStream fStream;
        private String fRestString;    // あるコマンドの範囲を超えて受け取った文字列は残しておく

        private String fOpponentName;

        public String getName() {
            if (fOpponentName == null) {
                return "Network Opponent";
            } else {
                return fOpponentName;
            }
        }
        public void newGame(int yourIndex) {
        }

        public void endGame(int result) {
        }

        public void setParam( int timeLimit, Action[] answerBox) {
        }

        public void makeAction(object currentState) {
        }

        // サーバに接続する
        public bool connect() {
            fSocket = new TcpClient();
            fRestString = "";
            fOpponentName = null;
            try {
                // ソケット接続
                fSocket.Connect(Settings.ConnectionIP, Settings.ConnectionPort);
                // ソケットストリーム取得
                fStream = fSocket.GetStream();
            } catch {
                // 接続失敗．大抵はサーバがないだけ．
                MessageBox.Show("Connection Failure");
                return false;
            }

            Logger.log("connected to server");
            return true;
        }

        // 接続後，６個の色をもらって開始できるようになるまでのやりとり
        public int[] waitUntilGetFirstStones(String playerName) {

            // 名前を聞かれるのを待つ
            waitFor("name");
            respond(playerName);  //返事する

            // 相手を教えてもらう
            fOpponentName = waitFor("opponent");
            respond("");
            Logger.netlog("Opponent name = " + fOpponentName);

            // 制限時間を教えてもらう
            String sLimitTime = waitFor("limitTime");
            respond("");
            Settings.LimitTime = Int32.Parse(sLimitTime);
            Logger.netlog("Seconds per a move = " + sLimitTime);

            // 最初の珠色を教えてもらう
            String sFirstColors = waitFor("firstColor");
            respond("");
            int[] cols = new int[sFirstColors.Length];
            for (int i = 0; i < sFirstColors.Length; i++) {
                cols[i] = Int32.Parse(sFirstColors.Substring(i, 1));
            }
            Logger.netlog("First Colors = " + sFirstColors);

            return cols;
        }

        // 次の色の珠を２個もらう．これが誰のものかは gameManager側で判断する
        public int[] waitUntilGetNexts() {
            String s = waitFor("nextColor");
            respond("");
            int[] cols = new int[2];
            for (int i = 0; i < 2; i++) {
                cols[i] = Int32.Parse(s.Substring(i, 1));
            }
            Logger.netlog("Next Colors = " + s);

            return cols;
        }

        // genmoveを待ち，答えを返す
        public void sendAction(Action a) {
            waitFor("genmove");
            respond(a.getX() + "" + a.getR());
            Logger.netlog("Send move " + a.toBriefString());
        }

        // genmoveを待ち，Stop/Resign を返す
        public void sendSpecialAction(int specialValue) {
            waitFor("genmove");
            respond("" + specialValue);
            Logger.netlog("Send value " + specialValue);
        }

        // 相手の手を待つ 位置と回転を表している場合と，-1/-2 の数字の場合がある
        public Action waitUntilOpponentAction(Action defaultAction) {
            String s = waitFor("play");
            respond("");
            if (s.Substring(0, 1).Equals("-") == true) {
                int val = Int32.Parse(s);
                return new Action(val);
            } else {
                int x = Int32.Parse(s.Substring(0, 1));
                int r = Int32.Parse(s.Substring(1, 1));

                Logger.netlog("Opponent move " + x + " " + Settings.RotateChars[r]);
                return new Action(defaultAction, x, r);
            }
        }

        // = ...\r\n の形で返事を送る
        private void respond(String message) {
            Byte[] dat = System.Text.Encoding.GetEncoding("UTF-8").GetBytes("= " + message + "\r\n");
            fStream.Write(dat, 0, dat.GetLength(0));
        }

        // 接続を切る
        public void waitForBeingClosed() {
            if (fSocket.Connected == true) {
                fStream.Close();
                fSocket.Close();
                Logger.log("Connection closed");
            }
        }




        // command...\r\n の形のものが来るまで待ち，...の部分を返す．
        // 複数のコマンドがちぎれたりひっついたりするので， \r\n の後ろを次の waitFor のために残す
        // closeされたら抜ける
        private String waitFor(String command) {
            command = command + " ";
            String s = fRestString;
            while (true) {
                Thread.Sleep(50);
                Application.DoEvents();

                if (Signal.FormClosed == true) return "";

                if (fSocket.Connected == false) {
                    Logger.log("Connection closed by server");
                    return "";
                }

                if (fSocket.Available > 0) {
                    Byte[] dat = new Byte[fSocket.Available];
                    fStream.Read(dat, 0, dat.GetLength(0));
                    s=s + System.Text.Encoding.GetEncoding("UTF-8").GetString(dat);
                }
                if (s.Length >= command.Length + 2) {
                    if (s.Substring(0, command.Length).Equals(command)) {
                        int pos = s.IndexOf("\r\n");
                        if (pos > 0) {
                            fRestString = s.Substring(pos + 2); // \r\n の後ろから全てを残す
                            return s.Substring(command.Length, pos - command.Length);  // 返すのは，commandの後ろから \r\n の前まで
                        }
                    }
                }
            }


        }





    }
}
