﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;

namespace PojeClient {
    // 対戦サーバ（といっても 1 vs 1 のみ）関係の仕事をするクラス
    // 将来的には Unixで動かしたいが，手の正当性検証などの問題もあるので Windowsで作る
    // ほぼこれのコピー：　http://homepage2.nifty.com/nonnon/SoftSample/CS.NET/SampleTcpIpSvr.html
    class VSServer {
        private TcpListener fListener;
        private Thread fServerThread;
        private ClientTcpIp[] fClients;
        public int fStatus = STATUS_DISABLED;

        public const int STATUS_DISABLED = 0;
        public const int STATUS_LISTEN = 1;
        public const int STATUS_READY = 2;

        private int[] fNexts1, fNexts2; // 次の珠の色


        // 対戦の受付を開始
        public void listenStart() {
            // IPアドレス＆ポート番号設定
            int myPort = Settings.ConnectionPort;
//            IPAddress myIp = IPAddress.Parse(Settings.ConnectionIP); 
            IPAddress myIp=null;
            foreach (IPAddress ip in Dns.GetHostAddresses(Dns.GetHostName())) {
                if (ip.AddressFamily == AddressFamily.InterNetwork) myIp = ip;
            }

            IPEndPoint myEndPoint = new IPEndPoint(myIp, myPort);
            Logger.log(myIp.ToString()+" listen at port "+myPort);


            // クライアント情報初期化
            fClients = new ClientTcpIp[2];

            // リスナー開始
            fListener = new TcpListener(myEndPoint);
            fListener.Start();

            // クライアント接続待ち開始
            fServerThread = new Thread(new ThreadStart(ServerThread));
            fServerThread.Start();

            fStatus = STATUS_LISTEN; 
        }



        // 名前を尋ねて受け取る
        public String askPlayerName(int idx) {
            sendCommand(idx, "name ");
            return waitForResponse(idx);
        }

        // 相手の名前を伝える
        public void sendOpponentName(int idx, String name) {
            sendCommand(idx, "opponent " + name);
            waitForResponse(idx);
        }

        // ルール，持ち時間，配珠など，共通の情報を通知する
        public void sendRules(int[] firsts) {
            for (int idx=0; idx<2; idx++) {
                sendCommand(idx, "limitTime "+Settings.LimitTime);
                waitForResponse(idx);  // ほんとは wait は非同期でまとめてでもいい

                String s = "";
                for (int i=0; i<firsts.Length; i++) s=s+firsts[i];
                sendCommand(idx, "firstColor "+s);
                waitForResponse(idx);
            }
        }

        // 次の色ストック関係，Phase 1 クリアする
        public void clearNexts() {
            fNexts1 = null;
            fNexts2 = null;
        }

        // Phase 2 新しいセットを加える
        public void prepareSendNexts(int[] nexts) {
            if (fNexts1 == null) {
                fNexts1 = nexts;
            } else {
                fNexts2 = nexts;
            }
        }

        // Phase 3 配る．ただし，idx=1 の人に対しては逆転して送る
        public void sendNexts() {
            if (fNexts1 == null) {
                // なにもしない
            } else if (fNexts2 == null) {
                // １つだけ送る
                sendCommand(0, "nextColor " + fNexts1[0] + "" + fNexts1[1]);
                waitForResponse(0);
                sendCommand(1, "nextColor " + fNexts1[0] + "" + fNexts1[1]);
                waitForResponse(1);
            } else {
                // 両方送る
                // idx=0 に対してはそのままの順番で
                sendCommand(0, "nextColor " + fNexts1[0] + "" + fNexts1[1]);
                waitForResponse(0);
                sendCommand(0, "nextColor " + fNexts2[0] + "" + fNexts2[1]);
                waitForResponse(0);
                // idx=1 に対しては逆の順番で
                sendCommand(1, "nextColor " + fNexts2[0] + "" + fNexts2[1]);
                waitForResponse(1);
                sendCommand(1, "nextColor " + fNexts1[0] + "" + fNexts1[1]);
                waitForResponse(1);
            }
        }

        // 着手を尋ねる．両方そろったら，格納のうえ，相手側に送る
        public void askActions(bool[] actionRequired, Action[][] answers) {
            // 必要なほうに尋ねる
            for (int idx = 0; idx < 2; idx++) {
                if (actionRequired[idx] == true) sendCommand(idx, "genmove ");
            }

            // Resign/Stopを送ってきた奴はそこで止まってる
            bool[] sentSpecial = new bool[2] { false, false };

            // そろうまで待つ．
            for (int idx = 0; idx < 2; idx++) {
                if (actionRequired[idx] == true) {
                    String s = waitForResponse(idx);
                    if (s.Substring(0, 1).Equals("-") == true) {
                        sentSpecial[idx] = true;
                        answers[idx][0] = new Action(Int32.Parse(s));
                    } else {
                        int x = Int32.Parse(s.Substring(0, 1));
                        int r = Int32.Parse(s.Substring(1, 1));
                        answers[idx][0] = new Action(answers[idx][0], x, r);
                    }
                }
            }

            // 相手側に送り，レスポンスを待つ
            // Resign/Stop を送ってきた奴は勝手に止まってるのでもう送らない
            for (int idx = 0; idx < 2; idx++) {
                if (actionRequired[idx] == true && sentSpecial[idx]==false) {
                    String s = "play " + answers[idx][0].getX() + "" + answers[idx][0].getR();
                    if (answers[idx][0].getValue() == Action.STOP || answers[idx][0].getValue() == Action.RESIGN ) {
                        s = "play "+answers[idx][0].getValue();
                    }
                    sendCommand(1 - idx, s);
                    waitForResponse(1 - idx);
                }
            }
        }





                




        #region 通信部

        // クライアント接続待ちスレッド
        private void ServerThread() {
            try {
                while (true) {
                    // ソケット接続待ち
                    TcpClient myTcpClient = fListener.AcceptTcpClient();
                    
                    // クライアントから接続有り．空いてるところを探す
                    int idx = -1;
                    for (int i = 1; i>=0; i--) {
                        if (fClients[i] == null) {
                            idx = i;
                        } else if( fClients[i].objSck.Connected == false){
                            idx = i;
                        }
                    }

                    if (idx == -1) {
                        myTcpClient.Close();  //空きがない （ここにくることは稀なはずだが一応）
                    } else {
                        // クライアント送受信オブジェクト生成
                        fClients[idx] = new ClientTcpIp();
                        fClients[idx].index = idx;
                        fClients[idx].fStringReceived = "";
                        fClients[idx].objSck = myTcpClient;
                        fClients[idx].objStm = myTcpClient.GetStream();
                        // クライアントとの送受信開始
                        Thread myClientThread = new Thread( new ThreadStart(fClients[idx].ReadWrite));
                        myClientThread.Start();

                        // 埋まった
                        if (idx == 1) {
                            fStatus = STATUS_READY;
                            break;
                        }
                    }
                }
            } catch {
                // fListener がStopされると多分 AcceptTcpClient() が例外を飛ばしてここにきてスレッドは終わるはず
                //System.Windows.Forms.MessageBox.Show("ServerThread end");
            }
        }




        // 受付を終了
        public void listenEnd() {
            fListener.Stop();
            fStatus = STATUS_DISABLED;
        }

        // クライアント切断
        public void closeConnection() {
            listenEnd();
            for (int i = 0; i < fClients.Length; i++) {
                if (fClients[i] != null ){
                    if (fClients[i].objSck.Connected == true) {
                        fClients[i].objStm.Close();
                        fClients[i].objSck.Close();
                    }
                }
            }
        }


        // command...\r\n の形でメッセージする
        private void sendCommand(int idx, String message) {
            Logger.netlog("send to "+idx+" : "+message);
            Byte[] dat = System.Text.Encoding.GetEncoding("UTF-8").GetBytes(message + "\r\n");
            fClients[idx].objStm.Write(dat, 0, dat.GetLength(0));
        }

        // = ...\r\n の形のものが来るまで待ち，...の部分を返す．
        // 複数のコマンドがちぎれたりひっついたりするので， \r\n の後ろを次の waitFor のために残す
        private String waitForResponse(int idx) {
            while (true) {
                Thread.Sleep(50);
                Application.DoEvents();
                if (Signal.FormClosed) return "";

                String s = fClients[idx].fStringReceived;
                if (s.Length >= 4) {
                    if (s.Substring(0, 2).Equals("= ")) {
                        int pos = s.IndexOf("\r\n");
                        if (pos > 0) {
                            fClients[idx].fStringReceived = s.Substring(pos + 2); // \r\n の後ろから全てを残す // Todo ロック
                            return s.Substring(2, pos - 2);  // 返すのは，commandの後ろから \r\n の前まで
                        }
                    }
                }
            }
        }

        #endregion
    }

    // クライアント受信クラス
    // ほぼこれのコピー：　http://homepage2.nifty.com/nonnon/SoftSample/CS.NET/SampleTcpIpSvr.html
    public class ClientTcpIp {
        public int index;
        public TcpClient objSck;
        public NetworkStream objStm;
        public String fStringReceived;  // 受け取った文字列を溜めておく

        // クライアント送受信スレッド
        public void ReadWrite() {
            try {
                while (true) {
                    // ソケット受信
                    Byte[] rdat = new Byte[1024];
                    int ldat = objStm.Read(rdat, 0, rdat.GetLength(0));
                    if (ldat > 0) {
                        // クライアントからの受信データ有り
                        Byte[] sdat = new Byte[ldat];
                        Array.Copy(rdat, sdat, ldat);
                        String msg = System.Text.Encoding.GetEncoding("UTF-8").GetString(sdat);
                        fStringReceived = fStringReceived + msg;  // 本当はこのへんロックしないといけないんじゃ
                        //MessageBox.Show("from client " + msg);
                    }else{
                        // ソケット切断有り
                        // ソケットクローズ
                        //MessageBox.Show("server; closing detected " + index);
                        objStm.Close();
                        objSck.Close();
                        return;
                    }
                }
            } catch {
            }
        }
    }

}
